"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
const enmap_1 = __importDefault(require("enmap"));
const ModeratorErrors_json_1 = __importDefault(require("./ModeratorErrors.json"));
const ModeratorError_1 = __importDefault(require("./ModeratorError"));
const Emitter_1 = __importDefault(require("./Emitter"));
const MutesManager_1 = __importDefault(require("./managers/MutesManager"));
const PunishmentsManager_1 = __importDefault(require("./managers/PunishmentsManager"));
const UtilsManager_1 = __importDefault(require("./managers/UtilsManager"));
class Moderator extends Emitter_1.default {
    client;
    errors;
    db;
    author;
    version;
    docs;
    ready;
    options;
    blacklist;
    mutes;
    punishments;
    utils;
    constructor(client, options) {
        super();
        this.options = {};
        if (!client)
            throw new ModeratorError_1.default(this.errors.requireClient);
        /**
         * Module Author
         *
         * @type {String}
        */
        this.author = require('../package.json').author;
        /**
         * Module Version
         *
         * @type {String}
        */
        this.version = require('../package.json').version;
        /**
         * Module Homepage
         *
         * @type {String}
        */
        this.docs = require('../package.json').homepage;
        /**
         * Moderator Database
        */
        this.db = {
            mutes: null,
            warns: null,
            blacklist: null
        };
        /**
         * Module Ready Status
         *
         * @type {Boolean}
        */
        this.ready = false;
        /**
         * Discord Client
         *
         * @type {Client}
         * @private
        */
        this.client = client;
        /**
         * Moderator Errors JSON
         *
         * @type {Object}
         * @private
        */
        this.errors = ModeratorErrors_json_1.default;
        /**
         * Moderator Mutes Manager
         *
         * @type {MutesManager}
        */
        this.mutes = new MutesManager_1.default(client, this);
        /**
         * Moderator Punishments Manager
         *
         * @type {PunishmentsManager}
        */
        this.punishments = new PunishmentsManager_1.default(client, this);
        /**
         * Moderator Utils Manager
         *
         * @type {UtilsManager}
        */
        this.utils = new UtilsManager_1.default(client, this);
        /**
         * Moderator Options
         *
         * @type {ModeratorOptions}
        */
        this.options = this.utils.fetchOptions();
        client.on('ready', () => {
            this.initModerator();
        });
    }
    /**
     * Method for module database initialization
     *
     * @private
    */
    initDB() {
        this.db.mutes = new enmap_1.default({
            name: this.options.muteConfig.name,
            dataDir: this.options.dirPath,
            wal: false
        });
        this.db.warns = new enmap_1.default({
            name: this.options.warnConfig.name,
            dataDir: this.options.dirPath,
            wal: false
        });
        this.db.blacklist = new enmap_1.default({
            name: this.options.blacklistConfig.name,
            dataDir: this.options.dirPath,
            wal: false
        });
    }
    /**
     * Method for module initialization
     *
     * @private
    */
    initModerator() {
        this.initDB();
        this.ready = true;
        this.emit('ready', this);
    }
}
module.exports = Moderator;
