import { Client } from 'discord.js';
import Enmap from 'enmap';

import ModeratorEmitter from '../src/Emitter';
import { Awaitable } from './other/Awaitable';

import ModeratorEvents from './ModeratorEvents';
import ModeratorOptions from './ModeratorOptions';
import ModeratorErrors from '../src/ModeratorErrors.json';
import { BlockData, MuteData, WarnData } from './other/DatabaseData';

import GuildBlacklistManager from './managers/GuildBlacklistManager';
import MutesManager from './managers/MutesManager';
import PunishmentsManager from './managers/PunishmentsManager';
import UtilsManager from './managers/UtilsManager';

declare class Moderator extends ModeratorEmitter {
    constructor(client: Client, options?: ModeratorOptions);

    private client: Client;
    private errors: typeof ModeratorErrors;

    public db: {
        mutes: Enmap<string, Array<MuteData>> | null;
        warns: Enmap<string, Array<WarnData>> | null;
        blacklist: Enmap<string, Array<BlockData>> | null;
    }

    public author: string;
    public version: string;
    public docs: string;

    public ready: boolean;
    public options: ModeratorOptions;

    public blacklist: GuildBlacklistManager;
    public mutes: MutesManager;
    public punishments: PunishmentsManager;
    public utils: UtilsManager;

    /**
     * Method for module database initialization
    */
    private initDB(): void;

    /**
     * Method for module initialization
    */
    private initModerator(): void;

    public on<K extends keyof ModeratorEvents>(event: K, listener: (...args: ModeratorEvents[K]) => void): this;
    public on<S extends string | symbol>(event: Exclude<S, keyof ModeratorEvents>, listener: (...args: any) => Awaitable<void>,): this;
    
    public once<K extends keyof ModeratorEvents>(event: K, listener: (...args: ModeratorEvents[K][]) => void): this;
    public once<S extends string | symbol>(event: Exclude<S, keyof ModeratorEvents>, listener: (...args: any[]) => Awaitable<void>): this;

    public emit<K extends keyof ModeratorEvents>(event: K, listener: (...args: ModeratorEvents[K][]) => void): boolean;
    public emit<S extends string | symbol> (event: Exclude<S, keyof ModeratorEvents>, listener: (...args: any[]) => void): boolean;
}

export = Moderator;