import { Client } from 'discord.js';

import Moderator from '../Moderator';

import { BlockData } from '../other/DatabaseData';

import ModeratorErrors from '../../src/ModeratorErrors.json';

declare class GuildBlacklistManager {
    constructor(client: Client, moderator: Moderator);

    private client: Client;
    private moderator: Moderator;
    private errors: typeof ModeratorErrors;

    /**
     * Method for issuing a lock to a user on the server
     * @param targetID Discord User ID
     * @param guildID Discord Guild ID
     * @param author Block author ID
     * @param reason Block reason
     * 
     * @returns Returns the lock issuance status and information about it
    */
    public add(targetID: string, guildID: string, author: string, reason?: string): Promise<{ status: boolean, data: BlockData }>

    /**
     * Method for checking if a user has a lock on the server
     * @param targetID Discord User ID
     * @param guildID Discord Guild ID
     * 
     * @returns Returns the validation status and lock information (if any)
    */
    public fetch(targetID: string, guildID: string): Promise<{ status: boolean, data: BlockData | null }>

    /**
     * Method to get user locks on all bot servers
     * @param targetID Discord User ID
     * 
     * @returns Returns the status of the search for locks and an array with them
    */
    public fetchAll(targetID: string): Promise<{ status: boolean, fetchUser: boolean, data: Array<BlockData> | null }>

    /**
     * Method to remove a user lock on the server
     * @param targetID Discord user ID
     * @param guildID Discord Guild ID
     * 
     * @returns Returns the user lock removal status
    */
    public remove(targetID: string, guildID: string): Promise<boolean>

    /**
     * Method to clear all locks on the server
     * @param guildID Discord Guild ID
     * 
     * @returns Returns the cleanup status
    */
    public clearGuild(guildID: string): Promise<boolean>

    /**
     * Method for completely clearing locks on all servers
     * 
     * @returns Returns the cleanup status
    */
    public clearAll(): Promise<boolean>
}

export = GuildBlacklistManager;