import { Client, Guild, GuildBan, GuildMember, TextBasedChannel } from 'discord.js';

import Moderator from '../Moderator';

import ModeratorErrors from '../../src/ModeratorErrors.json';

import { BanData, KickData, MuteData } from '../ModeratorData';

declare class PunishmentsManager {
    constructor(client: Client, moderator: Moderator);

    private client: Client;
    private moderator: Moderator;
    private errors: typeof ModeratorErrors;

    /**
     * Method for kicking a user from the server
     * @param target Discord GuildMember
     * @param author Kick Author ID
     * @param reason Kick Reason
     * 
     * @returns Returns the punishment status and information about it
    */
    public kick(target: GuildMember, author: string, reason?: string): Promise<{ status: boolean, data: KickData }>;
    
    /**
     * Method for banning a user from the server
     * @param target Discord GuildMember
     * @param author Ban Author ID
     * @param reason Ban Reason
     * 
     * @returns Returns the punishment status and information about it
    */
    public ban(target: GuildMember, author: string, reason?: string): Promise<{ status: boolean, data: BanData }>;
    
    /**
     * Method for unbanning a user on the server
     * @param targetID Discord User ID
     * @param guildID Discord Guild ID
     * 
     * @returns Returns the unbanned status and information about the ban
    */
    public unban(targetID: string, guildID: string): Promise<{ status: boolean, data: GuildBan }>;
    
    /**
     * Method to punish the user
     * @param target Discord GuildMember
     * @param channel Discord Text Channel
     * @param author Punishment Author ID
     * @param roleID Punishment Role ID
     * 
     * @returns Returns the punishment status and information about it
    */
    public set(target: GuildMember, channel: TextBasedChannel, author: string, roleID?: string): Promise<{ status: boolean, type: string, data: MuteData | KickData | BanData }>;
}

export = PunishmentsManager;