import { Client, Guild, GuildMember, PermissionsString, User } from 'discord.js';

import Moderator from '../Moderator';

import ModeratorOptions from '../ModeratorOptions';
import ModeratorErrors from '../../src/ModeratorErrors.json';

declare class UtilsManager {
    constructor(client: Client, moderator: Moderator);

    private client: Client;
    private moderator: Moderator;
    private errors: typeof ModeratorErrors;

    /**
     * Method for getting a random number
     * @param min Minimum vakue
     * @param max Maximum value
     * 
     * @returns Random number in a given range
    */
    public getRandomNumber(min: number, max: number): number;

    /**
     * Method for getting a random string
     * @param data Array with random strings
     * 
     * @returns Random string
    */
    public getRandomString(data: Array<string>): string;

    /**
     * Method for fetching Moderator options
     * 
     * @returns Options for stable operation of the module
    */
    public fetchOptions(): ModeratorOptions;

    /**
     * Method for checking member existence
     * @param guild Discord Guild
     * @param target Discord GuildMember or Discord User ID
     * 
     * @returns Member verification status and information about him
    */
    public fetchMember(guild: Guild, target: string | GuildMember): { status: boolean, user: User | null };

    /**
     * Method for checking client permissions on the server
     * @param guild Discord Guild
     * @param permissions Permissions Array
     * 
     * @returns Boolean value that will indicate the presence/absence of permissions
    */
    public fetchClientPermissions(guild: Guild, permissions: Array<PermissionsString>): boolean;

    /**
     * Method for checking member permissions on the server
     * @param member Discord GuildMember
     * @param permissions Permissions Array
     * 
     * @returns Boolean value that will indicate the presence/absence of permissions
    */
    public fetchMemberPermissions(member: GuildMember, permissions: Array<PermissionsString>): boolean;
}

export = UtilsManager;